﻿using System;
using System.Drawing;

namespace i384
{
    public struct Vector2
    {
        public float X;

        public float Y;

        public float Length
        {
            get
            {
                return (float)Math.Sqrt(this.X * this.X + this.Y * this.Y);
            }
        }

        public Vector2(float value)
        {
            this.X = value;
            this.Y = value;
        }

        public Vector2(float x, float y)
        {
            this.X = x;
            this.Y = y;
        }

        public Vector2(double x, double y)
        {
            this.X = (float)x;
            this.Y = (float)y;
        }

        public Vector2(Point point)
        {
            this.X = point.X;
            this.Y = point.Y;
        }

        public static implicit operator PointF(Vector2 vector)
        {
            return new PointF(vector.X, vector.Y);
        }

        public static implicit operator SizeF(Vector2 vector)
        {
            return new SizeF(vector.X, vector.Y);
        }

        public static bool operator ==(Vector2 left, Vector2 right)
        {
            return left.X == right.X && left.Y == right.Y;
        }

        public static bool operator !=(Vector2 left, Vector2 right)
        {
            return !(left == right);
        }

        public static Vector2 operator +(Vector2 left, Vector2 right)
        {
            return new Vector2(left.X + right.X, left.Y + right.Y);
        }

        public static Vector2 operator -(Vector2 left, Vector2 right)
        {
            return new Vector2(left.X - right.X, left.Y - right.Y);
        }

        public static Vector2 operator -(Vector2 vector)
        {
            return new Vector2() - vector;
        }

        public static Vector2 operator *(Vector2 left, Vector2 right)
        {
            return new Vector2(left.X * right.X, left.Y * right.Y);
        }

        public static Vector2 operator *(Vector2 left, float right)
        {
            return new Vector2(left.X * right, left.Y * right);
        }

        public static Vector2 operator *(float left, Vector2 right)
        {
            return new Vector2(left * right.X, left * right.Y);
        }

        public static Vector2 operator /(Vector2 left, Vector2 right)
        {
            return new Vector2(left.X / right.X, left.Y / right.Y);
        }

        public static Vector2 operator /(Vector2 left, float right)
        {
            return new Vector2(left.X / right, left.Y / right);
        }

        public static Vector2 operator /(float left, Vector2 right)
        {
            return new Vector2(left / right.X, left / right.Y);
        }

        public static float Distance(Vector2 a, Vector2 b)
        {
            Vector2 difference = a - b;
            return Math.Abs(difference.Length);
        }

        public void Normalize()
        {
            float length = this.Length;
            this.X /= length;
            this.Y /= length;
        }

        public Vector2 DirectionUnit
        {
            get
            {
                return Vector2.Normalize(this);
            }
        }

        public static Vector2 Normalize(Vector2 vector)
        {
            vector.Normalize();
            return vector;
        }

        public Vector2 Rotate(float angle)
        {
            return new Vector2(this.X * Math.Cos(angle) - this.Y * Math.Sin(angle),
                this.X * Math.Sin(angle) + this.Y * Math.Cos(angle));
        }

        public override bool Equals(object obj)
        {
            if (obj is Vector2)
            {
                Vector2 other = (Vector2)obj;
                return this == other;
            }

            return false;

        }

        public override int GetHashCode()
        {
            int prime = 13;
            int result = 1;
            unchecked
            {
                result = result * prime + this.X.GetHashCode();
                result = result * prime + this.Y.GetHashCode();
            }

            return result;
        }

        public override string ToString()
        {
            return string.Format("X:{0} Y:{1}", this.X, this.Y);
        }
    }

}
