﻿using System.Drawing;

namespace i384
{
    /// <summary>
    /// Represents a drawable indexed vertex of a graph.
    /// </summary>
    public class Vertex
    {
        /// <summary>
        /// The drawing position of the vertex relative to the drawing surface.
        /// </summary>
        public Vector2 Position { get; set; }

        public Vertex(Vector2 position)
        {
            this.Position = position;
        }

        /// <summary>
        /// Draws the vertex on the specified drawing surface.
        /// </summary>
        /// <param name="graphics">The drawing surface where the vertex is to be drawn.</param>
        /// <param name="number">The number that refers to this particular vertex.</param>
        public void Draw(Graphics graphics, int number, Color color)
        {
            RectangleF layoutRectangle = new RectangleF(this.Position, DrawingSettings.CircleSize);

            graphics.DrawEllipse(new Pen(color, 2.0f), layoutRectangle);

            graphics.DrawString(number.ToString(), DrawingSettings.Font, new Pen(color, 2.0f).Brush, layoutRectangle, new StringFormat
            {
                Alignment = StringAlignment.Center,
                LineAlignment = StringAlignment.Center
            });
        }
    }
}
